/* $Id: init.c,v 1.5 1999/06/08 00:38:20 marcus Exp $
******************************************************************************

   Bse extension initialization.

   Copyright (C) 1999 Andreas Beck	[becka@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <string.h>

#include "bse_int.h"


/* Static variables 
 */

static int 	 _ggibseLibIsUp=0;
static void	*_ggibseConfigHandle;
static char      ggibseconfstub[512]=BSECONFFILE;
static char     *ggibseconffile=ggibseconfstub+BSETAGLEN;

/* Extension ID. Defaulting to -1 should make segfault on abuse more likely...
 */
EXPORTVAR ggi_extid ggiBseID = -1;

static int changed(ggi_visual_t vis,int whatchanged)
{
	switch(whatchanged) {
	case GGI_CHG_APILIST: {
		int i;
		const char *fname;
		ggi_dlhandle *lib;
		char api[256], args[256];

		for(i=0; ggiGetAPI(vis, i, api, args) == 0; i++) {
			strcat(api,"-bse");
			printf("trying %s.\n",api);
			fname = ggMatchConfig(_ggibseConfigHandle, api, NULL);
			if (fname == NULL) {
				/* No special implementation for this sublib */
				continue;
			}
			lib = ggiExtensionLoadDL(vis, fname, args, NULL);
		}
	}
	break;
	}
                                
	return 0;
}

static int _ggi_error(void) { return -1; }

int ggiBseInit(void)
{
	int err;

	_ggibseLibIsUp++;
	if (_ggibseLibIsUp > 1) return 0; /* Initialize only at first call. */

	err = ggLoadConfig(ggibseconffile, &_ggibseConfigHandle);
	if (err != GGI_OK) {
		fprintf(stderr,"LibGGIBSE: couldn't open %s\n",
			ggibseconffile);
		_ggibseLibIsUp--;
		return err;
	}

	ggiBseID = ggiExtensionRegister("GGIBSE", sizeof(struct bse_ext),
					changed);

	if (ggiBseID < 0) {
		fprintf(stderr,
			"LibGGIBSE: failed to register as extension.\n");
		_ggibseLibIsUp--;
		ggFreeConfig(_ggibseConfigHandle);
		return ggiBseID;
	}

	return 0;
}

int ggiBseExit(void)
{
	int rc;
    
	if (!_ggibseLibIsUp) return -1;

	if (_ggibseLibIsUp > 1) {
		/* Exit only at last call */
		_ggibseLibIsUp--;
		return 0;
	}
	
	rc = ggiExtensionUnregister(ggiBseID);
	ggFreeConfig(_ggibseConfigHandle);

	_ggibseLibIsUp = 0;

	return rc;
}

int ggiBseAttach(ggi_visual_t vis)
{
	int rc;

	rc = ggiExtensionAttach(vis, ggiBseID);
	if (rc == 0) {
		/* We are actually creating the primary instance. */
		memset(LIBGGI_BSEEXT(vis), 0, sizeof(ggibseext));
		LIBGGI_BSEEXT(vis)->alloc=
			(void *) _ggi_error;
		LIBGGI_BSEEXT(vis)->first=NULL;
                        
		/* Now fake an "API change" so the right libs get loaded */
		changed(vis, GGI_CHG_APILIST);
	}

	return rc;
}

int ggiBseDetach(ggi_visual_t vis)
{
	ggibse_t next;
	ggibse_t remains=LIBGGI_BSEEXT(vis)->first;

	while(remains) {
		next=remains->next;
		ggiBseFree(remains);
		remains=next;
	}
	
	return ggiExtensionDetach(vis, ggiBseID);
}
